import nl.saxion.app.CsvReader;
import nl.saxion.app.SaxionApp;

import java.awt.*;
import java.util.ArrayList;

public class Application implements Runnable {

    public static void main(String[] args) {
        SaxionApp.start(new Application(), 800, 768);
    }

    ArrayList<Star> starList = new ArrayList<>();

    public void run() {
        SaxionApp.turnBorderOff(); //don't draw a border

        boolean isRunning = true;
        while(isRunning){
            //print menu
            SaxionApp.clear();
            SaxionApp.printLine("=========================");
            SaxionApp.printLine("Welcome to the Night Sky!");
            SaxionApp.printLine("=========================");
            SaxionApp.printLine("1. Read stars from stars.csv");
            SaxionApp.printLine("2. Print all stars");
            SaxionApp.printLine("3. Find the brightest star");
            SaxionApp.printLine("4. Show the Night Sky");
            SaxionApp.printLine("0. Exit");
            SaxionApp.print("What would you like to do? ");
            //SaxionApp.saveImage("resources/mainMenu.png");
            int userInput = SaxionApp.readInt();

            if(userInput == 1){
                loadStarData();
            }else if(userInput == 2){
                if (!starList.isEmpty()) {
                    printAllStars();
                }else {
                    SaxionApp.printLine("No stars available yet!", Color.RED);
                }
            }else if(userInput == 3){
                if (!starList.isEmpty()) {
                    String brightestStar = findBrightestStar();
                    SaxionApp.printLine("The brightest star is: " + brightestStar);
                    //SaxionApp.saveImage("resources/brightestStar.png");
                }else {
                    SaxionApp.printLine("No stars available yet!", Color.RED);
                }
            }else if(userInput == 4){
                if (!starList.isEmpty()) {
                    SaxionApp.clear();
                    showNightSky();
                }else {
                    SaxionApp.printLine("No stars available yet!", Color.RED);
                }
            }else if(userInput == 0) {
                SaxionApp.printLine("Bye bye!");
                isRunning = false;
            }else {
                SaxionApp.printLine("Not a valid option, please try again", Color.RED);
            }
            SaxionApp.pause();

        }
    }


    public void printAllStars(){
        for (Star star : starList) {
            SaxionApp.printLine(star.name + " distance: " + star.distance + " luminosity: " + star.luminosity);
        }
    }

    public void loadStarData(){
        starList = new ArrayList<>();
        CsvReader reader = new CsvReader("stars.csv");
        reader.setSeparator(',');
        reader.skipRow();
        while(reader.loadRow()){
            starList.add(createStar(
                    reader.getInt(0),
                    reader.getInt(1),
                    reader.getString(2),
                    reader.getDouble(3),
                    reader.getInt(4),
                    reader.getString(5)
            ));
        }
        SaxionApp.printLine("All stars are loaded!");
    }

    public String findBrightestStar(){
        Star brightestStar = starList.get(0);
        double highest = calculateBrightness(brightestStar);
        for(Star star: starList){
            double brightness = calculateBrightness(star);
            if(brightness > highest){
                brightestStar = star;
                highest = brightness;
            }
        }
        return brightestStar.name;
    }

    public void showNightSky(){
        for(Star star: starList){
            int rad = (int)calculateBrightness(star);
            SaxionApp.setFill(getColor(star.direction));
            SaxionApp.drawCircle(star.x, star.y, rad);
            SaxionApp.drawText(star.name, star.x -rad , star.y + rad, 18);
        }
    }

    //---------------------- helper methodes ----------------------
    public Star createStar(int x, int y, String name, double distance, int luminosity, String direction){
        Star star = new Star();
        star.x = x;
        star.y = y;
        star.name = name;
        star.distance = distance;
        star.luminosity = luminosity;
        star.direction = direction;

        return star;
    }


    public double calculateBrightness(Star star){
        return star.luminosity / star.distance;
    }

    public Color getColor(String direction){
        if(direction.equals("TU")){
            return Color.blue;
        }else if(direction.equals("ST")){
            return Color.yellow;
        }else{
            return Color.red;
        }
    }


}



